import os
from typing import List
from ctestgen.generator import Var, Enum, Include, Define, ControlStructure, Call
from ctestgen.generator.style_parametres import indent_char, indent_size


class Program:
    def __init__(self, name):
        self.name = name
        self.includes = []
        self.defines = []
        self.enums = []
        self.global_variables = []
        self.functions = []

    def __str__(self):
        program_code = '//Generated by ctestgen generator.\n'
        for include in self.includes:
            program_code += include.get_declaration()
        if self.includes:
            program_code += '\n'

        for define in self.defines:
            program_code += define.get_declaration()
        if self.defines:
            program_code += '\n'

        for enum in self.enums:
            program_code += enum.get_declaration()
        if self.enums:
            program_code += '\n'

        for global_var in self.global_variables:
            program_code += global_var.get_declaration() + ';\n'
        if self.global_variables:
            program_code += '\n'

        for func in self.functions:
            program_code += func.get_declaration()
        return program_code

    def set_name(self, name):
        self.name = name

    def add_include(self, include):
        if isinstance(include, Include):
            self.includes.append(include)

    def add_define(self, define):
        if isinstance(define, Define):
            self.defines.append(define)

    def add_enum(self, enum):
        if isinstance(enum, Enum):
            self.enums.append(enum)

    def add_global_variable(self, global_var):
        if isinstance(global_var, Var):
            self.global_variables.append(global_var)

    def add_function(self, func):
        if isinstance(func, Function):
            self.functions.append(func)

    def write_to_file(self, output_dir):
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)
        with open(output_dir + '/' + self.name + '.c', 'wt') as output_file:
            output_file.write(str(self))


class CodeBlock:
    def __init__(self, *body):
        if body and isinstance(body[0], List):
            body = body[0]
        self.body = body

    def __str__(self):
        block_code = ''
        for expression in self.body:
            if isinstance(expression, ControlStructure):
                block_code += str(expression)
            else:
                block_code += str(expression) + ';\n'
        # Transformations for auto indentation
        block_strings = block_code.split('\n')
        if not block_strings[len(block_strings) - 1]:
            block_strings = block_strings[:len(block_strings) - 1]
        block_code = '\n'.join(map(lambda s: indent_size*indent_char + s, block_strings))
        block_code = '{\n' + block_code + '\n}\n'
        return block_code


class Function:
    def __init__(self, name, return_type, *args):
        if args and isinstance(args[0], List):
            args = args[0]
        self.name = name
        self.return_type = return_type
        self.arguments = args
        self.body = CodeBlock()

    def set_body(self, code_block: CodeBlock):
        assert isinstance(code_block, CodeBlock), 'Types mismatch'
        self.body = code_block

    def get_declaration(self):
        function_code = str(self.return_type) \
                        + ' ' + self.name + '('
        add_comma = False
        for argument in self.arguments:
            if isinstance(argument, Var):
                if add_comma:
                    function_code += ', '
                else:
                    add_comma = True
                function_code += argument.get_declaration()

        function_code += ') ' + str(self.body) + '\n'
        return function_code

    def __str__(self):
        return self.name

    def __call__(self, *args):
        if args and isinstance(args[0], List):
            args = args[0]
        return Call(self, *args)



import os
import json
from typing import List
from ctestgen.generator import For, Assignment, IsLess, Increment, ArrayType, Var, \
    CodeBlock, Define, EnumConstant, ConstVar, ArrayC99, Int


def pass_array(array, subscript_template, assignment_source=20, iter_var_type=Int):
    assert isinstance(array.var_type, ArrayType), \
        'Array types mismatch'

    if not iter_var_type:
        iter_var_type = array.var_type.base_type

    iter_var = Var('i0', iter_var_type)
    iter_vars = [iter_var]
    first_loop = For(Assignment(iter_var.get_declaration(), 0),
                     IsLess(iter_var, array.var_type.dimension_sizes[0]),
                     Increment(iter_var))
    prev_loop = first_loop
    for current_dim in range(1, len(array.var_type.dimension_sizes)):
        iter_var = Var('i' + str(current_dim), iter_var_type)
        iter_vars.append(iter_var)
        current_loop = For(Assignment(iter_var.get_declaration(), 0),
                           IsLess(iter_var, array.var_type.dimension_sizes[current_dim]),
                           Increment(iter_var))
        prev_loop.set_body(CodeBlock(current_loop))
        prev_loop = current_loop

    subscript_expr = subscript_template.generate_subscript(array, iter_vars)
    pass_expr = (Assignment(subscript_expr, assignment_source))
    prev_loop.set_body(CodeBlock(pass_expr))

    return first_loop


class BitVector:
    def __init__(self, num_system_base):
        self.num_system_base = num_system_base
        self.vector = 0

    def set_vector(self, vector):
        self.vector = vector

    def get_bit(self, i):
        return (self.vector // (self.num_system_base ** i)) % self.num_system_base


def write_arrays_to_json(name, output_dir, *arrays, is_function_argument=False):
    if arrays and isinstance(arrays[0], List):
        arrays = arrays[0]
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
    with open(os.path.join(output_dir, name + '.ans'), 'wt') as output_file:
        result_map = {'Sizes': {}, 'Accesses': {}}
        for array in arrays:
            result_map['Accesses'][array.name] = \
                [template.get_str_coefficients() for template in array.subscript_templates]
            array_sizes = []
            # contains_var_size = False
            for dim_size in array.var_type.dimension_sizes:
                if isinstance(dim_size, EnumConstant):
                    array_sizes.append(str(dim_size.value))
                elif isinstance(dim_size, ConstVar):
                    array_sizes.append(str(dim_size.init_value))
                elif isinstance(dim_size, Var):
                    array_sizes.append(dim_size.name)
                elif isinstance(dim_size, Define):
                    array_sizes.append(dim_size.process_macros())
                else:
                    array_sizes.append(str(dim_size))
            if is_function_argument or isinstance(array, ArrayC99) or \
                    (not is_function_argument and isinstance(array.var_type.dimension_sizes[0], Var)):
                array_sizes[0] = '***COULDNOTCOMPUTE***'
            result_map['Sizes'][array.name] = array_sizes

        output_file.write(json.dumps(result_map))


def generate_dim_sizes_names(dims_count):
    predefined_names = ['N', 'M', 'L', 'K', 'O']
    if dims_count <= len(predefined_names):
        return predefined_names[:dims_count]
    return ['N' + str(i) for i in range(dims_count)]



from abc import ABCMeta, abstractmethod
from ctestgen.generator import Add, Mul, Subscript


class DimSubscriptTemplate(metaclass=ABCMeta):
    @abstractmethod
    def generate_subscript(self, iter_var):
        pass

    @abstractmethod
    def get_coefficients(self):
        pass

    @abstractmethod
    def get_str_coefficients(self):
        pass

    def __str__(self):
        return str(self.get_coefficients())


class LinearDimSubscriptTemplate:
    """Template for generating subscript expression of form a*i + b"""

    def __init__(self, multiply_coefficient=0, add_coefficient=0):
        self.multiply_coefficient = multiply_coefficient
        self.add_coefficient = add_coefficient

    def generate_subscript(self, iter_var):
        if self.add_coefficient:
            if self.multiply_coefficient:
                return Add(Mul(iter_var, self.multiply_coefficient), self.add_coefficient)
            else:
                return str(self.add_coefficient)
        else:
            if self.multiply_coefficient:
                return Mul(iter_var, self.multiply_coefficient)
            else:
                return '0'

    def get_coefficients(self):
        return self.multiply_coefficient, self.add_coefficient

    def get_str_coefficients(self):
        return str(self.multiply_coefficient), str(self.add_coefficient)

    def __str__(self):
        return 'Mul coefficient: ' + str(self.multiply_coefficient) + \
               ' add coefficient: ' + str(self.add_coefficient)


class ArraySubscriptTemplate:
    def __init__(self, dim_templates):
        self.dim_templates = dim_templates

    def generate_subscript(self, array, iter_vars):
        assert len(self.dim_templates) == len(iter_vars), \
            'Dimensions count and given iterative variables count mismatch.' + str(self.dim_templates) + ' ' + str(iter_vars)
        array.subscript_templates.append(self)
        subscript_arguments = [array] + [self.dim_templates[i].generate_subscript(iter_vars[i])
                                         for i in range(len(self.dim_templates))]
        return Subscript(subscript_arguments)

    def get_coefficients(self):
        return [template.get_coefficients() for template in self.dim_templates]

    def get_str_coefficients(self):
        return [template.get_str_coefficients() for template in self.dim_templates]


class ArrayLinearSubscriptTemplate(ArraySubscriptTemplate):
    def __init__(self, args):
        """ArrayLinearSubscriptTemplate((1,3), (5,7), (9,11))"""
        super().__init__([LinearDimSubscriptTemplate(coefficients[0], coefficients[1]) for coefficients in args])



from abc import ABCMeta, abstractmethod


class ControlStructure(metaclass=ABCMeta):
    @abstractmethod
    def __str__(self):
        pass


class Loop(ControlStructure):
    @abstractmethod
    def set_body(self, code_block):
        pass

    @abstractmethod
    def __str__(self):
        pass


class For(Loop):
    def __init__(self, init, continue_condition, step, body=None):
        self.init = init
        self.continue_condition = continue_condition
        self.step = step
        self.body = body

    def set_body(self, code_block):
        self.body = code_block

    def __str__(self):
        return 'for (' + str(self.init) + \
                   '; ' + str(self.continue_condition) + \
                   '; ' + str(self.step) + \
                   ') ' + str(self.body)


class While(Loop):
    def __init__(self, continue_condition, body=None):
        self.continue_condition = continue_condition
        self.body = body

    def set_body(self, code_block):
        self.body = code_block

    def __str__(self):
        return 'while (' + str(self.continue_condition) + ') ' + str(self.body)


class If(ControlStructure):
    def __init__(self, condition, true_body=None, else_body=None):
        self.condition = condition
        self.true_body = true_body
        self.else_body = else_body

    def set_true_body(self, code_block):
        self.true_body = code_block

    def set_else_body(self, code_block):
        self.else_body = code_block

    def __str__(self):
        if_code = 'if (' + str(self.condition) + ') ' + str(self.true_body)
        if self.else_body:
            # drop last line break
            if_code = if_code[:(len(if_code)-1)]
            # if_code += (self.else_body.indent_level - 1)*indent_size*indent_char + 'else' + str(self.else_body)
            if_code += ' else ' + str(self.else_body)
        return if_code



import re
from typing import List
from datetime import datetime
from ctestgen.generator.c_operators import Call


class Include:
    def __init__(self, header_name, is_local_file=False):
        self.header_name = header_name
        self.is_local_file = is_local_file

    def get_declaration(self):
        include_code = '#include '
        if self.is_local_file:
            include_code += '"' + self.header_name + '"'
        else:
            include_code += '<' + self.header_name + '>'
        include_code += '\n'
        return include_code

    def __str__(self):
        return self.get_declaration()


class Define:
    def __init__(self, macro_title, macro_body):
        self.macro_title = macro_title
        self.macro_body = str(macro_body)
        assert re.fullmatch(
            r'(?:[_a-zA-Z][_a-zA-Z0-9]*)(?:\((?:[_a-zA-Z][_a-zA-Z0-9]*)(?:, *(?:[_a-zA-Z][_a-zA-Z0-9]*))*\))?',
            self.macro_title), \
            'Incorrect define title'

        identifiers = re.findall(
            r'(?:[_a-zA-Z][_a-zA-Z0-9]*)', self.macro_title)
        self.macro_name = identifiers[0]
        self.macro_arguments = identifiers[1:]

    def get_declaration(self):
        define_code = '#define ' + \
                      self.macro_title + ' ' + \
                      str(self.macro_body) + '\n'
        return define_code

    def process_macros(self, *args):
        if args and isinstance(args[0], List):
            args = args[0]
        assert len(args) == len(self.macro_arguments), \
            'Macros arguments mismatch'
        processed_body = self.macro_body
        processed_macro_arguments = list()
        replacing_protection = str(datetime.now())
        for i in range(len(args)):
            processed_body = processed_body.replace(self.macro_arguments[i], self.macro_arguments[i] +
                                                    replacing_protection)
            processed_macro_arguments.append(self.macro_arguments[i] + replacing_protection)
        for i in range(len(args)):
            processed_body = processed_body.replace(processed_macro_arguments[i], str(args[i]))
        return processed_body

    def __str__(self):
        return self.macro_name

    def __call__(self, *args):
        if args and isinstance(args[0], List):
            args = args[0]
        return Call(self.macro_name, *args)



import subprocess
import os
import sys
from datetime import datetime
import pathlib
import enum


def find_tests(base_dir, test_filename_extensions):
    """
    Returns dictionary that collects pairs of
    directory and test filenames from it.
    """
    tests = dict()
    for dir_description in os.walk(base_dir):
        dir_filenames = dir_description[2]
        target_filenames = set()
        if len(dir_filenames) != 0:
            for filename in dir_filenames:
                file_extension = ''.join(pathlib.Path(filename).suffixes)
                if file_extension in test_filename_extensions:
                    target_filenames.add(filename)
            if len(target_filenames) != 0:
                tests[dir_description[0]] = target_filenames
    return tests


def _get_datetime_string(current_datetime):
    return current_datetime.strftime('%Y_%m_%d_%H_%M_%S')


def _get_actual_datetime_string():
    return _get_datetime_string(datetime.now())


def init_output_dir(output_dir_base, runner_name):
    output_dir = os.path.join(output_dir_base, runner_name + '_' + _get_actual_datetime_string())
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
    return output_dir


def _init_output_text_file(filename, tests, output_dir):
    output_file_path = os.path.join(output_dir, filename)
    with open(output_file_path, 'w') as output_file:
        for test_dir in tests.keys():
            output_file.write(test_dir + '\n')
    return output_file_path


def init_success_output_file(tests, output_dir):
    success_output_filename = 'success.txt'
    return _init_output_text_file(success_output_filename, tests, output_dir)


def init_fail_output_file(tests, output_dir):
    fail_output_filename = 'fail.txt'
    return _init_output_text_file(fail_output_filename, tests, output_dir)


def init_metrics_output_file(tests, output_dir):
    metrics_output_filename = 'metrics.txt'
    return _init_output_text_file(metrics_output_filename, tests, output_dir)


def find_compiler_environment_variables():
    compiler_environment_variables = dict()
    if sys.platform == 'win32':
        vswhere_str = '"' + os.path.join('%ProgramFiles(x86)%', 'Microsoft Visual Studio', 'Installer', 'vswhere.exe') \
                      + '"\n'

        cmd_call = ["cmd", "/q", "/k", "echo off"]
        vswhere_process = subprocess.Popen(cmd_call, stdout=subprocess.PIPE, stdin=subprocess.PIPE,
                                           universal_newlines=True, shell=True)
        vswhere_process.stdin.write(vswhere_str)
        vswhere_process.stdin.flush()
        vswhere_process_out, vswhere_process_err = vswhere_process.communicate()
        vswhere_process.kill()
        install_path = ''
        for line in vswhere_process_out.splitlines():
            if 'installationPath:' in line:
                install_path = line[line.find(':') + 2:]
                break

        dev_cmd_path = '"' + install_path + '\Common7\Tools\VsDevCmd.bat"\n'
        dev_cmd_process = subprocess.Popen(cmd_call, stdout=subprocess.PIPE, stdin=subprocess.PIPE,
                                           universal_newlines=True, shell=True)
        dev_cmd_process.stdin.write(dev_cmd_path)
        dev_cmd_process.stdin.write('SET\n')
        dev_cmd_process_out, dev_cmd_process_err = dev_cmd_process.communicate()
        dev_cmd_process.kill()
        for line in dev_cmd_process_out.splitlines():
            if '=' in line:
                var_description = line.split('=', 1)
                compiler_environment_variables[var_description[0]] = var_description[1]
    return compiler_environment_variables


def get_program_response(args, env):
    process = subprocess.run(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                             universal_newlines=True, env=env)
    return process.stdout, process.stderr


def _get_timedelta_string(delta):
    seconds = delta.total_seconds()
    hours = seconds // 3600
    seconds = seconds % 3600
    minutes = seconds // 60
    seconds = seconds % 60
    return str(hours) + 'h - ' + str(minutes) + 'm - ' + str(seconds) + 's'


def get_metrics_description(metrics):
    metrics_description = ''
    for metric, value in metrics.items():
        if metric != 'start_time' and metric != 'finish_time':
            metrics_description += '\t' + metric + ': ' + str(value) + '\n'
    if 'start_time' in metrics.keys() and 'finish_time' in metrics.keys():
        delta = metrics['finish_time'] - metrics['start_time']
        metrics_description += '\ttime: ' + _get_timedelta_string(delta) + '\n'
    return metrics_description


class TestRunResult:
    class ResultType(enum.Enum):
        FAIL = 0
        SUCCESS = 1
        
    def __init__(self, result_type, test_output):
        self.result_type = result_type
        self.test_output = test_output




from abc import abstractmethod
from typing import List
from ctestgen.generator import TestGenerator, generate_dim_sizes_names


class ArrayTestGenerator(TestGenerator):
    def __init__(self, output_dir, *dims_count_set):
        super().__init__(output_dir)
        if dims_count_set and isinstance(dims_count_set[0], List):
            dims_count_set = dims_count_set[0]
        self.dim_sizes_names_set = [generate_dim_sizes_names(dims_count) for dims_count in dims_count_set]

    @abstractmethod
    def _generate_programs(self):
        pass



from ctestgen.generator.c_macros import *
from ctestgen.generator.c_types import *
from ctestgen.generator.c_control_structures import *
from ctestgen.generator.c_operators import *
from ctestgen.generator.c_control_structures import *
from ctestgen.generator.c_structure_elements import *
from ctestgen.generator.c_std_functions import *
from ctestgen.generator.c_std_constants import *
from ctestgen.generator.utils import *
from ctestgen.generator.templates import *
from ctestgen.generator.generator import *
from ctestgen.generator.arrays_test_generator import *



from abc import ABCMeta, abstractmethod
from datetime import datetime

from ctestgen.runner import find_tests,\
    get_metrics_description, TestRunResult, init_fail_output_file, \
    init_success_output_file, init_output_dir, init_metrics_output_file


class TestRunner(metaclass=ABCMeta):
    def __init__(self,
                 test_base_dir,
                 runner_name=None,
                 output_base_dir='runner_output',
                 test_filename_extensions=None,
                 dump_results_to_files=True,
                 print_run_progress=True,
                 print_percent_step=10,
                 print_global_metrics=True,
                 print_testsets_metrics=True):
        self.test_base_dir = test_base_dir
        self.runner_name = runner_name if runner_name is not None else str(self.__class__)
        self.output_base_dir = output_base_dir
        self.test_filename_extensions = test_filename_extensions \
            if test_filename_extensions is not None else ['.c']
        self.global_metrics = dict()
        self.testsets_metrics = dict()
        self.dump_results_to_files = dump_results_to_files
        self.print_run_progress = print_run_progress
        self.print_global_metrics = print_global_metrics
        self.print_percent_step = print_percent_step
        self.print_testsets_metrics = print_testsets_metrics
        self.output_dir = None
        self.success_output_file_path = None
        self.fail_output_file_path = None
        self.metrics_output_file_path = None

    def _on_run_start(self, tests):
        pass

    def _on_run_finish(self, tests):
        pass

    def _on_testdir(self, test_dir, test_filenames):
        pass

    @staticmethod
    def _filter_test_filenames(test_filenames):
        return test_filenames

    def _get_env(self):
        return None

    @abstractmethod
    def _on_test(self, test_dir, test_filename, env) -> TestRunResult:
        pass

    def run(self):
        tests = find_tests(self.test_base_dir, self.test_filename_extensions)

        if self.dump_results_to_files:
            self.output_dir = init_output_dir(self.output_base_dir, self.runner_name)
            self.success_output_file_path = init_success_output_file(tests, self.output_dir)
            self.fail_output_file_path = init_fail_output_file(tests, self.output_dir)
            self.metrics_output_file_path = init_metrics_output_file(tests, self.output_dir)

        env = self._get_env()

        self._on_run_start(tests)

        self.global_metrics['tests_count'] = 0
        self.global_metrics['success_count'] = 0
        self.global_metrics['fail_count'] = 0
        self.global_metrics['success_tests'] = list()
        self.global_metrics['fail_tests'] = list()
        self.global_metrics['start_time'] = datetime.now()
        self.global_metrics['finish_time'] = None

        for test_dir in tests.keys():
            self.testsets_metrics[test_dir] = dict()
            self.testsets_metrics[test_dir]['success_count'] = 0
            self.testsets_metrics[test_dir]['fail_count'] = 0
            self.testsets_metrics[test_dir]['tests_count'] = 0
            self.testsets_metrics[test_dir]['success_tests'] = list()
            self.testsets_metrics[test_dir]['fail_tests'] = list()
            self.testsets_metrics[test_dir]['start_time'] = None
            self.testsets_metrics[test_dir]['finish_time'] = None

        for test_dir, test_filenames in tests.items():
            self.testsets_metrics[test_dir]['start_time'] = datetime.now()
            self.testsets_metrics[test_dir]['tests_count'] = len(test_filenames)
            self._on_testdir(test_dir, test_filenames)
            test_filenames = self._filter_test_filenames(test_filenames)

            current_progress_percent = 0
            test_idx = 0
            for test_filename in test_filenames:
                if self.print_run_progress:
                    if test_idx / len(test_filenames) * 100 >= current_progress_percent:
                        print('\t\t\t\t\t' + str(current_progress_percent) + '%')
                        current_progress_percent += self.print_percent_step
                    test_idx += 1

                test_result = self._on_test(test_dir, test_filename, env)
                if test_result.result_type == TestRunResult.ResultType.SUCCESS:
                    self.testsets_metrics[test_dir]['success_count'] += 1
                    self.testsets_metrics[test_dir]['success_tests'].append(test_filename)
                    if self.dump_results_to_files:
                        with open(self.success_output_file_path, 'a') as success_output_file:
                            success_output_file.write('\nTest: ' + test_filename + '\n' +
                                                      test_result.test_output + '\n')
                else:
                    self.testsets_metrics[test_dir]['fail_count'] += 1
                    self.testsets_metrics[test_dir]['fail_tests'].append(test_filename)
                    if self.dump_results_to_files:
                        with open(self.fail_output_file_path, 'a') as fail_output_file:
                            fail_output_file.write('\nTest: ' + test_filename + '\n' +
                                                   test_result.test_output + '\n')

            if self.print_run_progress:
                print('\t\t\t\t\t100%')
            self.testsets_metrics[test_dir]['finish_time'] = datetime.now()
            self.global_metrics['tests_count'] += self.testsets_metrics[test_dir]['tests_count']
            self.global_metrics['success_count'] += self.testsets_metrics[test_dir]['success_count']
            self.global_metrics['fail_count'] += self.testsets_metrics[test_dir]['fail_count']
            self.global_metrics['success_tests'] += self.testsets_metrics[test_dir]['success_tests']
            self.global_metrics['fail_tests'] += self.testsets_metrics[test_dir]['fail_tests']

        self.global_metrics['finish_time'] = datetime.now()

        self._on_run_finish(tests)

        testsets_metrics_description = ''
        for test_dir in tests.keys():
            testsets_metrics_description += 'Testdir: ' + test_dir + '\n'
            testsets_metrics_description += get_metrics_description(self.testsets_metrics[test_dir])
        if self.print_testsets_metrics:
            print(testsets_metrics_description)

        global_metrics_description = 'Global:\n' + \
                                     get_metrics_description(self.global_metrics)
        if self.print_global_metrics:
            print(global_metrics_description)

        if self.dump_results_to_files:
            if self.print_testsets_metrics:
                with open(self.metrics_output_file_path, 'a') as metrics_output_file:
                    metrics_output_file.write(testsets_metrics_description + '\n')
            if self.print_global_metrics:
                with open(self.metrics_output_file_path, 'a') as metrics_output_file:
                    metrics_output_file.write(global_metrics_description + '\n')



import os
from abc import abstractmethod
from ctestgen.runner import TestRunner, get_program_response, \
    find_compiler_environment_variables, TestRunResult


class BasicTestRunner(TestRunner):
    def __init__(self, run_arguments, *args, print_test_info=True, **kwargs):
        super().__init__(*args, **kwargs)
        self.run_arguments = run_arguments
        self.print_test_info = print_test_info

    def _get_env(self):
        return find_compiler_environment_variables()

    @abstractmethod
    def _process_program_response(self, test_dir, test_filename, program_response) -> TestRunResult:
        pass

    def _on_test(self, test_dir, test_filename, env):
        if self.print_test_info:
            print(test_filename)
        program_response = get_program_response(self.run_arguments + [os.path.join(test_dir, test_filename)], env)
        return self._process_program_response(test_dir, test_filename, program_response)



from typing import Tuple, List
from ctestgen.generator import Define
from ctestgen.generator.style_parametres import indent_size, indent_char


class EnumConstant:
    def __init__(self, name, value=None):
        self.name = name
        self.value = value

    def get_declaration(self):
        enum_constant_code = self.name
        if self.value:
            enum_constant_code += ' = ' + str(self.value)
        return enum_constant_code

    def __str__(self):
        return self.name


class Enum:
    def __init__(self, *constants, name=None):
        """ Usage like Enum(('K', 7), ('M', 8), 'G')"""
        if isinstance(constants[0], List):
            constants = constants[0]
        self.constants = [EnumConstant(c[0], c[1]) if isinstance(c, Tuple) else EnumConstant(c) for c in constants]
        self.name = name

    def _get_title(self):
        return 'enum ' + self.name if self.name else 'enum'

    def get_declaration(self):
        enum_code = self._get_title() + ' {'
        add_comma = False
        for constant in self.constants:
            if add_comma:
                enum_code += ',\n' + indent_size * indent_char
            else:
                enum_code += '\n' + indent_size * indent_char
                add_comma = True

            enum_code += constant.get_declaration()

        enum_code += '\n};\n\n'
        return enum_code

    def __str__(self):
        return self._get_title()

    def __getitem__(self, item):
        return self.constants[item]

    def __iter__(self):
        return self.constants.__iter__()


class Type:
    def __init__(self, typename):
        self.typename = typename

    def get_var_declaration(self, var_name):
        return self.typename + ' ' + var_name

    def __call__(self, name):
        return Var(name, self)

    def __str__(self):
        return self.typename


Char = Type('char')
Int = Type('int')
UInt = Type('unsigned int')
Long = Type('long')
SizeT = Type('size_t')
Float = Type('float')
Double = Type('double')
Void = Type('void')
Bool = Type('bool')


class Pointer(Type):
    def __init__(self, inner_type: Type):
        super().__init__(str(inner_type) + '*')
        self.inner_type = inner_type


class ConstType(Type):
    def __init__(self, inner_type: Type):
        super().__init__('const ' + str(inner_type))
        self.inner_type = inner_type

    def __call__(self, name, init_value=None):
        return ConstVar(name, self.inner_type, init_value)


ConstChar = ConstType(Char)
ConstInt = ConstType(Int)
ConstUInt = ConstType(UInt)
ConstLong = ConstType(Long)
ConstSizeT = ConstType(SizeT)
ConstFloat = ConstType(Float)
ConstDouble = ConstType(Double)
ConstVoid = ConstType(Void)
ConstBool = ConstType(Bool)


class ArrayType(Type):
    def __init__(self, base_type: Type, *dimension_sizes):
        super().__init__(str(base_type))
        if isinstance(dimension_sizes[0], List):
            dimension_sizes = dimension_sizes[0]
        self.dimension_sizes = dimension_sizes
        self.base_type = base_type

    def get_var_declaration(self, var_name):
        var_declaration = self.typename + ' ' + var_name
        for dim_size in self.dimension_sizes:
            var_declaration += '['
            if dim_size:
                if isinstance(dim_size, Var):
                    var_declaration += dim_size.name
                elif isinstance(dim_size, EnumConstant):
                    var_declaration += dim_size.name
                elif isinstance(dim_size, Define):
                    var_declaration += dim_size.macro_title
                else:
                    var_declaration += str(dim_size)
            var_declaration += ']'
        return var_declaration


class ArrayTypeC99(ArrayType):
    def get_var_declaration(self, var_name):
        var_declaration = self.typename + \
                          ' (*' + var_name + ')'
        for i in range(1, len(self.dimension_sizes)):
            dim_size = self.dimension_sizes[i]
            var_declaration += '['
            if dim_size:
                if isinstance(dim_size, Var):
                    var_declaration += dim_size.name
                elif isinstance(dim_size, EnumConstant):
                    var_declaration += dim_size.name
                elif isinstance(dim_size, Define):
                    var_declaration += dim_size.macro_title
            var_declaration += ']'
        return var_declaration


class Var:
    def __init__(self, name, var_type: Type):
        self.name = name
        self.var_type = var_type

    def get_declaration(self):
        return self.var_type.get_var_declaration(self.name)

    def __str__(self):
        return self.name


class ConstVar(Var):
    def __init__(self, name, base_type: Type, init_value=None):
        super().__init__(name, ConstType(base_type))
        self.init_value = init_value

    def get_declaration(self):
        declaration_code = self.var_type.get_var_declaration(self.name)
        if self.init_value:
            declaration_code += ' = ' + str(self.init_value)
        return declaration_code


class Array(Var):
    def __init__(self, name, base_type, *dim_sizes):
        super().__init__(name, ArrayType(base_type, *dim_sizes))
        self.subscripts = []
        self.subscript_templates = []


class ArrayC99(Var):
    def __init__(self, name, base_type, *dim_sizes):
        super().__init__(name, ArrayTypeC99(base_type, *dim_sizes))
        self.subscripts = []
        self.subscript_templates = []



import re
from typing import List


class Operator:
    def __init__(self, operator_definition, is_repeatable=False):
        self.operator_definition = operator_definition
        # counting format fields in operator definition
        self.arguments_count = len(re.findall(r'\{[^\}]*\}', operator_definition))
        self.is_repeatable = is_repeatable

    def generate_operator_code(self, *args):
        if args and isinstance(args[0], List):
            args = args[0]
        if len(args) == self.arguments_count:
            return self.operator_definition.format(args=args)
        if self.is_repeatable and self.arguments_count == 2:
            # recursive call
            return self.operator_definition.format(args=(args[0], self.generate_operator_code(*args[1:])))
        assert False, 'Arguments count mismatch'

    def __call__(self, *args):
        return self.generate_operator_code(*args)


Assignment = Operator('{args[0]} = {args[1]}')

Add = Operator('{args[0]} + {args[1]}', True)
Sub = Operator('{args[0]} - {args[1]}', True)
Mul = Operator('{args[0]} * {args[1]}', True)
Div = Operator('{args[0]} / {args[1]}', True)
Modulo = Operator('{args[0]} % {args[1]}', True)

Increment = Operator('{args[0]}++')
IncrementPrefix = Operator('++{args[0]}')
Decrement = Operator('{args[0]}--')
DecrementPrefix = Operator('--{args[0]}')

IsEqual = Operator('{args[0]} == {args[1]}')
IsNotEqual = Operator('{args[0]} != {args[1]}')
IsGreater = Operator('{args[0]} > {args[1]}')
IsGreaterOrEqual = Operator('{args[0]} >= {args[1]}')
IsLess = Operator('{args[0]} < {args[1]}')
IsLessOrEqual = Operator('{args[0]} <= {args[1]}')

Not = Operator('!{args[0]}')
And = Operator('{args[0]} && {args[1]}', True)
Or = Operator('{args[0]} || {args[1]}', True)

BitNot = Operator('~{args[0]}')
BitAnd = Operator('{args[0]} & {args[1]}', True)
BitOr = Operator('{args[0]} | {args[1]}', True)
BitXor = Operator('{args[0]} ^ {args[1]}', True)
BitLeftShift = Operator('{args[0]} << {args[1]}', True)
BitRightShift = Operator('{args[0]} >> {args[1]}', True)

AddAssignment = Operator('{args[0]} += {args[1]}')
SubAssignment = Operator('{args[0]} -= {args[1]}')
MultAssignment = Operator('{args[0]} *= {args[1]}')
DivAssignment = Operator('{args[0]} /= {args[1]}')
ModuloAssignment = Operator('{args[0]} %= {args[1]}')
BitAndAssignment = Operator('{args[0]} &= {args[1]}')
BitOrAssignment = Operator('{args[0]} |= {args[1]}')
BitXorAssignment = Operator('{args[0]} ^= {args[1]}')
BitLeftShiftAssignment = Operator('{args[0]} <<= {args[1]}')
BitRightShiftAssignment = Operator('{args[0]} >>= {args[1]}')


class SubscriptOperator(Operator):
    def __init__(self):
        """Arguments count is dynamically changed value, there is no meaningful value for it"""
        super().__init__('[]')

    def __call__(self, *args):
        if args and isinstance(args[0], List):
            args = args[0]
        assert (len(args) > 1), \
            'Arguments count in call not match operator`s minimal arguments count'
        args[0].subscripts.append(args[1:])
        self.arguments_count = len(args)
        self.operator_definition = '{args[0]}'
        for i in range(1, len(args)):
            self.operator_definition += '[{args[' + str(i) + ']}]'
        return self.generate_operator_code(*args)


Subscript = SubscriptOperator()

Indirection = Operator('*{args[0]}')
AddressOf = Operator('&{args[0]}')
StructureReference = Operator('{args[0]}.{args[1]}', True)
StructureDereference = Operator('{args[0]}->{args[1]}', True)


class _CallOperator(Operator):
    def __init__(self):
        """Arguments count is dynamically changed value, there is no meaningful value for it"""
        super().__init__('()')

    def __call__(self, *args):
        if args and isinstance(args[0], List):
            args = args[0]
        assert (len(args) > 1), \
            'Arguments count in call not match operator`s minimal arguments count'
        self.arguments_count = len(args)
        self.operator_definition = '{args[0]}('
        for i in range(1, len(args)):
            if i != 1:
                self.operator_definition += ', '
            self.operator_definition += '{args[' + str(i) + ']}'
        self.operator_definition += ')'
        return self.generate_operator_code(*args)


Call = _CallOperator()

TernaryCondition = Operator('{args[0]} ? {args[1]} : {args[2]}')

Conversion = Operator('({args[0]}) {args[1]}')

SizeOf = Operator('sizeof({args[0]})')


class _VarDeclarationOperator(Operator):
    def __init__(self):
        super().__init__('{args[0]}')

    def __call__(self, var):
        return self.generate_operator_code(var.get_declaration())


VarDeclaration = _VarDeclarationOperator()

Return = Operator('return {args[0]}')
Break = Operator('break')
Continue = Operator('continue')



from setuptools import setup, find_packages
import ctestgen

with open('README.md', 'r') as readme_file:
    long_description = readme_file.read()

setup(
    name='ctestgen',
    version=ctestgen.__version__,
    author='Vasilkin Vladislav',
    author_email='volandtymim@gmail.com',
    description='https://github.com/VolandTymim/ctestgen',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/VolandTymim/ctestgen',
    keywords='python c code-generation code-generator autotesting',
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ]
)



from abc import ABCMeta, abstractmethod
from typing import List
import os

from ctestgen.generator import Program


class TestGenerator(metaclass=ABCMeta):
    def __init__(self, output_dir):
        self.output_dir = output_dir

    @abstractmethod
    def _generate_programs(self) -> List[Program]:
        pass

    def run(self):
        if not os.path.exists(self.output_dir):
            os.makedirs(self.output_dir)
        programs = self._generate_programs()
        for program in programs:
            program.write_to_file(self.output_dir)
